#!/usr/bin/perl

#┌─────────────────────────────
#│ Diary Note : diary.cgi - 2022/03/28
#│ copyright (c) KentWeb
#│ http://www.kent-web.com/
#└─────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│  Diary Note v5.1 : - 2022/03/28
#│  Edited by TRANSFORM Since 2012
#│  https://www.transformsite.com/
#└─────────────────────────────────
# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use lib "./lib";
use CGI::Minimal;
use File::ReadBackwards;
use Calendar::Japanese::Holiday;

# 設定ファイル認識
require "./init.cgi";
my %cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
my %in = parse_form($cgi);

# 処理分岐
if ($in{mode} eq "read")  { read_log(); }
if ($in{mode} eq "popup") { popup(); }
if ($in{mode} eq "news")  { news_log(); }
if ($in{mode} eq 'album') { album_page(); }
if ($in{mode} eq 'find') { find_data(); }
top_menu();

#-----------------------------------------------------------
#  TOP画面
#-----------------------------------------------------------
sub top_menu {
	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/top.html") or error("open err: top.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置き換え
	$tmpl =~ s/!cgi_title!/$cf{cgi_title}/g;
	$tmpl =~ s/!homepage!/$cf{homepage}/g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ /(.+)<!-- loop_begin -->(.+)<!-- loop_end -->(.+)/s
			? ($1,$2,$3)
			: error("テンプレート不正");

	# ヘッダ
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	# index読み取り
	open(IN,"$cf{datadir}/index.dat") or error("open err: index.dat");
	while (<IN>) {
		my @f = split(/,/);

		my $tmp = $loop;
		$tmp =~ s/!year!/$f[0]/g;

		foreach my $i (1 .. 12) {
			if ($f[$i]) {
				$tmp =~ s|!$i!|<a href="$cf{diary_cgi}?mode=read&amp;y=$f[0]&amp;m=$i">$i月</a>|g;
			} else {
				$tmp =~ s|!$i!|$i月|g;
			}
		}

		print $tmp;
	}
	close(IN);

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  内容閲覧
#-----------------------------------------------------------
sub read_log {
	# 現在の年月
	my ($mday,$mon,$year) = (localtime())[3 .. 5];
	$year += 1900;
	$mon++;
	$in{y} =~ s/\D//g;
	$in{m} =~ s/\D//g;
	if ($in{y} < 1970 or $in{m} < 1 or $in{m} > 12) {
		$in{y} = $year;
		$in{y} = $mon;
	}

	# ページ数
	my $pg = $in{pg} || 0;

	# テンプレート読込
	open(IN,"$cf{tmpldir}/read.html") or error("open err: read.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置き換え
	$tmpl =~ s/!cgi_title!/$cf{cgi_title}/g;
	$tmpl =~ s/!year!/$in{y}/g;
	$tmpl =~ s/!month!/$in{m}/g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s|!icon:(\w+)\.(\w+)!|<img src="$cf{iconurl}/$1.$2" alt="$1" class="icon" />|g;

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
				? ($1,$2,$3)
				: error("テンプレート不正");

	# ログ展開（ログが存在しない場合はエラーにしない）
	my $log = sprintf("%04d%02d",$in{y},$in{m});
	my ($body,$i,%cal);
	open(IN,"$cf{logdir}/$log.txt");
	while (<IN>) {
		$i++;
		my ($no,$d,$wk,$sub,$msg,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tube) = split(/<>/);
		$cal{$d}++;
		next if ($i < $pg + 1);
		next if ($i > $pg + $cf{pg_max});

		# 日付フォーマット
		my $date = "$in{y}年$in{m}月$d日（$cf{week}[$wk]）";

		# 自動リンク
		$msg = auto_link($msg) if ($cf{autolink});
		$msg =~ s/&amp;/&/g;

	# YouTube
	if ($cf{tube}) {
		$tube =~ s|!([\w\-]{1,})!|<div class="imageCover"><a href="https://www.youtube.com/embed/$1" class="lightview" data-lightview-type="iframe" data-lightview-options="width: 845,height: 480, viewport: 'scale'"><img src="https://i.ytimg.com/vi/$1/mqdefault.jpg" width="240" height="154" class="baseImage"></a><a href="https://www.youtube.com/embed/$1" class="lightview" data-lightview-type="iframe" data-lightview-options="width: 845,height: 480, viewport: 'scale'"><img src="$cf{tube_ico}" width="64" height="64" class="coverImage"></a></div>|g;
	}

		# 画像
		my $image;
		if ($e1) { $image .= image(1,$no,$e1,$w1,$h1,$sub); }
		if ($e2) { $image .= image(2,$no,$e2,$w2,$h2,$sub); }
		if ($e3) { $image .= image(3,$no,$e3,$w3,$h3,$sub); }

		my $tmp = $loop;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s/!day-title!/$sub/g;
		$tmp =~ s/!comment!/$msg/g;
		$tmp =~ s/<!-- image -->/$image/g;
		$tmp =~ s/<!-- youtube -->/$tube/ if ($tube);

		$body .= $tmp;
	}
	close(IN);

	$foot =~ s/!page-btn!/make_pgbtn($i,$pg)/e;
	$foot =~ s|<!-- calen -->(.+?)<!-- /calen -->|put_calen($1,$mday,$mon,$year,\%cal)|se;

	# 先月/翌月ボタン
	my ($b_y,$b_m,$n_y,$n_m) = get_ym($in{y},$in{m});
	$foot =~ s|!back-btn!|<a href="$cf{diary_cgi}?mode=read&amp;y=$b_y&amp;m=$b_m"><img src="$cf{iconurl}/back.png" alt="前月" /></a>|;
	$foot =~ s|!next-btn!|<a href="$cf{diary_cgi}?mode=read&amp;y=$n_y&amp;m=$n_m"><img src="$cf{iconurl}/next.png" alt="翌月" /></a>|;

	# 表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head, $body;
	footer($foot);
}

#-----------------------------------------------------------
#  新着情報
#-----------------------------------------------------------
sub news_log {
	
	# 現在の年月
	my ($mday,$mon,$year) = (localtime())[3 .. 5];
	$mon = sprintf("%02d",++$mon);
	$year += 1900;
	
		# 年月引数
		$in{y} =~ s/\D//g;
		$in{m} =~ s/\D//g;
		$in{d} =~ s/\D//g;
		
		if ($in{y} < 1970 or $in{m} < 1 or $in{m} > 12) {
			$in{y} = $year;
			$in{m} = $mon;
			$in{d} = $mday;
		}

	# データ読み込み
	my ($ttl,$log,$ym,$pgs) = $in{mode} eq 'news' ? read_news() : $in{y} && $in{m} && $in{d};
	
	# テンプレート読込
	open(IN,"$cf{tmpldir}/news.html") or error("open err: news.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	# 文字置き換え
	$tmpl =~ s/!ttl-name!/$ttl/g;
	$tmpl =~ s/!cgi_title!/$cf{cgi_title}/g;
	$tmpl =~ s/!year!/$in{y}/g;
	$tmpl =~ s/!month!/$in{m}/g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s|!icon:(\w+)\.(\w+)!|<img src="$cf{iconurl}/$1.$2" alt="$1" class="icon" />|g;
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ /(.+)<!-- loop_begin -->(.+)<!-- loop_end -->(.+)/s
			? ($1,$2,$3)
			: error("テンプレート不正");
	
	# ヘッダ
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	my $i = 0;
	foreach (@{$log}) {
		my ($no,$d,$wk,$sub,$msg,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tube) = split(/<>/);

		$i++;
		my ($y,$m) = $$ym{$i} =~ /^(\d{4})(\d{2})$/ && ($1,$2);

		# 日付フォーマット
		my $date = "$y年$m月$d日（$cf{week}[$wk]）";
		
		# 自動リンク
		$msg = auto_link($msg) if ($cf{autolink});

	# YouTube
	if ($cf{tube}) {
		$tube =~ s|!([\w\-]{1,})!|<div class="imageCover"><a href="https://www.youtube.com/embed/$1" class="lightview" data-lightview-type="iframe" data-lightview-options="width: 845,height: 480, viewport: 'scale'"><img src="https://i.ytimg.com/vi/$1/mqdefault.jpg" width="240" height="154" class="baseImage"></a><a href="https://www.youtube.com/embed/$1" class="lightview" data-lightview-type="iframe" data-lightview-options="width: 845,height: 480, viewport: 'scale'"><img src="$cf{tube_ico}" width="64" height="64" class="coverImage"></a></div>|g;
	}

		# 画像
		my $image;
		if ($e1) { $image .= image(1,$no,$e1,$w1,$h1,$sub); }
		if ($e2) { $image .= image(2,$no,$e2,$w2,$h2,$sub); }
		if ($e3) { $image .= image(3,$no,$e3,$w3,$h3,$sub); }

		my $tmp = $loop;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s/!day-title!/$sub/g;
		$tmp =~ s/!comment!/$msg/g;
		$tmp =~ s/<!-- image -->/$image/g;
		$tmp =~ s/<!-- youtube -->/$tube/ if ($tube);
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  新着記事読み込み
#-----------------------------------------------------------
sub read_news {
	# 月次ファイル読み取り
	my @tmp;
	opendir(DIR,"$cf{logdir}");
	while( my $dir = readdir(DIR) ) {
		next if ($dir !~ /^(\d{6})\.txt$/);
		
		push(@tmp,$1);
	}
	closedir(DIR);
	
	# データ順次読み込み
	my ($i,$flg,@log,%ym);
	for my $log ( sort{ $b <=> $a } @tmp ) {
		my $bw = File::ReadBackwards->new("$cf{logdir}/$log.txt") or next;
		while( defined( my $line = $bw->readline ) ) {
			my ($no,$d,$wk,$sub,$msg,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef) = split(/<>/,$line);
			next if ($msg eq '');
			
			$i++;
			$ym{$i} = $log;
			push(@log,$line);
			
			# 最大記事
			if ($i >= $cf{new_data}) {
				$flg++;
				last;
			}
		}
		last if ($flg);
	}
	
	return('新着記事',\@log,\%ym);
}

#-----------------------------------------------------------
#  アルバム機能
#-----------------------------------------------------------
sub album_page {
	# ページ数定義
	my $pg = $in{pg} || 0;

	# 現在の年月
	my ($mday,$mon,$year) = (localtime())[3 .. 5];
	$mon = sprintf("%02d",++$mon);
	$year += 1900;
	
		# 年月引数
		$in{y} =~ s/\D//g;
		$in{m} =~ s/\D//g;
		$in{d} =~ s/\D//g;
		
		if ($in{y} < 1970 or $in{m} < 1 or $in{m} > 12) {
			$in{y} = $year;
			$in{m} = $mon;
			$in{d} = $mday;
		}

	# データ読み込み
	my ($ttl,$log,$ym,$pgs) = $in{mode} eq 'album' ? read_album() : $in{y} && $in{m} && $in{d};
			
	# ペイジャー
	my $album_pager = $pgs ? album_pager($pgs,$in{pg},"&mode=album") : '';
	
	# テンプレート読込
	open(IN,"$cf{tmpldir}/album.html") or error("open err: album.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	# 文字置き換え
	$tmpl =~ s/!ttl-name!/$ttl/g;
	$tmpl =~ s/!cgi_title!/$cf{cgi_title}/g;
	$tmpl =~ s/!year!/$in{y}/g;
	$tmpl =~ s/!month!/$in{m}/g;
	$tmpl =~ s/!pager!/$album_pager/g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s|!icon:(\w+)\.(\w+)!|<img src="$cf{iconurl}/$1.$2" alt="$1" class="icon" />|g;
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ /(.+)<!-- loop_begin -->(.+)<!-- loop_end -->(.+)/s
			? ($1,$2,$3)
			: error("テンプレート不正");

	# ヘッダ
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	my $i = 0;
	foreach (@{$log}) {
		my ($no,$d,$wk,$sub,$msg,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tube) = split(/<>/);
		
		$i++;
		my ($y,$m) = $$ym{$i} =~ /^(\d{4})(\d{2})$/ && ($1,$2);

		# 日付フォーマット
		my $date = "$y年$m月$d日（$cf{week}[$wk]）";

		next if (!$e1);

		# 画像
		my $image;
		if ($e1) { $image .= album_image(1,$no,$e1,$w1,$h1,$sub); }
		if ($e2) { $image .= album_image(2,$no,$e2,$w2,$h2,$sub); }
		if ($e3) { $image .= album_image(3,$no,$e3,$w3,$h3,$sub); }

		my $tmp = $loop;
		$tmp =~ s/!day!/$date/g;
		$tmp =~ s/!image!/$image/g;
		$tmp =~ s/!caption!/$sub/g;
		print $tmp;
	}
	
	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  アルバムデータ読み込み
#-----------------------------------------------------------
sub read_album {

	# ページ数定義
	my $pg = $in{pg} || 0;

	# 月次ファイル読み取り
	my @tmp;
	opendir(DIR,"$cf{logdir}");
	while( my $dir = readdir(DIR) ) {
		next if ($dir !~ /^(\d{6})\.txt$/);
		
		push(@tmp,$1);
	}
	closedir(DIR);
	
	# データ順次読み込み
	my ($i,$n,$flg,@log,%ym);
	for my $log ( sort{ $b <=> $a } @tmp ) {
		my $bw = File::ReadBackwards->new("$cf{logdir}/$log.txt") or next;
		while( defined( my $line = $bw->readline ) ) {
			my ($no,$d,$wk,$sub,$msg,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef) = split(/<>/,$line);

			if ($msg ne '') {
				$i++;
				next if ($i < $pg + 1);
				next if ($i > $pg + $cf{album_data});
				
				$n++;
				$ym{$n} = $log;
				push(@log,$line);
			}
		}
	}
	
	return('アルバム一覧',\@log,\%ym,$i);
}

#-----------------------------------------------------------
#  ワード検索
#-----------------------------------------------------------
sub find_data {

	# ページ数定義
	my $pg = $in{pg} || 0;

	# 現在の年月
	my ($mday,$mon,$year) = (localtime())[3 .. 5];
	$mon = sprintf("%02d",++$mon);
	$year += 1900;
	
		# 年月引数
		$in{y} =~ s/\D//g;
		$in{m} =~ s/\D//g;
		$in{d} =~ s/\D//g;
		
		if ($in{y} < 1970 or $in{m} < 1 or $in{m} > 12) {
			$in{y} = $year;
			$in{m} = $mon;
			$in{d} = $mday;
		}
	
	# 条件
	$in{cond} =~ s/\D//g;
	$in{word} =~ s|<br />||g;

	# 検索条件プルダウン
	my %op = (1 => 'AND', 0 => 'OR');
	my $op_cond;
	foreach (1,0) {
		if ($in{cond} eq $_) {
			$op_cond .= qq|<option value="$_" selected>$op{$_}\n|;
		} else {
			$op_cond .= qq|<option value="$_">$op{$_}\n|;
		}
	}

	# 検索実行
	my ($ttl,$hit,$find,$ym) = $in{mode} eq 'find' ? read_find($in{word},$in{cond}) : $in{y} && $in{m} && $in{d};
	
	# ページ繰越ボタン作成
	my $find_pager = find_pager($hit,$in{pg},"&mode=find");
	
	# テンプレート読込
	open(IN,"$cf{tmpldir}/find.html") or error("open err: find.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	# 文字置き換え
	$tmpl =~ s/!ttl-name!/$ttl/g;
	$tmpl =~ s/!cgi_title!/$cf{cgi_title}/g;
	$tmpl =~ s/!year!/$in{y}/g;
	$tmpl =~ s/!month!/$in{m}/g;
	$tmpl =~ s/<!-- op_cond -->/$op_cond/;
	$tmpl =~ s/!word!/$in{word}/;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s|!icon:(\w+)\.(\w+)!|<img src="$cf{iconurl}/$1.$2" alt="$1" class="icon" />|g;
	if ($in{word}) {
		$tmpl =~ s/!result!/$hit/g;
		$tmpl =~ s/<!-- pager -->/$find_pager/g;
	} else {
		$tmpl =~ s/<!-- find_begin -->.+<!-- find_end -->//s;
	}
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ /(.+)<!-- loop_begin -->(.+)<!-- loop_end -->(.+)/s
			? ($1,$2,$3)
			: error("テンプレート不正");
	
	# ヘッダ
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	my $i = 0;
	foreach (@{$find}) {
		my ($no,$d,$wk,$sub,$msg,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tube) = split(/<>/);
		
		$i++;
		my ($y,$m) = $$ym{$i} =~ /^(\d{4})(\d{2})$/ && ($1,$2);
		
		# 日付フォーマット
		my $date = "$y年$m月$d日（$cf{week}[$wk]）";
		
		# 自動リンク
		$msg = auto_link($msg) if ($cf{autolink});
	
	# YouTube
	if ($cf{tube}) {
		$tube =~ s|!([\w\-]{1,})!|<div class="imageCover"><a href="https://www.youtube.com/embed/$1" class="lightview" data-lightview-type="iframe" data-lightview-options="width: 845,height: 480, viewport: 'scale'"><img src="https://i.ytimg.com/vi/$1/mqdefault.jpg" width="240" height="154" class="baseImage"></a><a href="https://www.youtube.com/embed/$1" class="lightview" data-lightview-type="iframe" data-lightview-options="width: 845,height: 480, viewport: 'scale'"><img src="$cf{tube_ico}" width="64" height="64" class="coverImage"></a></div>|g;
	}
		
		# 画像
		my $image;
		if ($e1) { $image .= image(1,$no,$e1,$w1,$h1,$sub); }
		if ($e2) { $image .= image(2,$no,$e2,$w2,$h2,$sub); }
		if ($e3) { $image .= image(3,$no,$e3,$w3,$h3,$sub); }
		
		my $tmp = $loop;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s/!day-title!/$sub/g;
		$tmp =~ s/!comment!/$msg/g;
		$tmp =~ s/<!-- image -->/$image/g;
		$tmp =~ s/<!-- youtube -->/$tube/ if ($tube);
		print $tmp;
	
	}
	
	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  ワード検索データ読み込み
#-----------------------------------------------------------
sub read_find {
	my ($y,$m,$d) = @_;
	my ($word,$cond) = @_;

	# キーワードを配列化
	$word =~ s/　/ /g;
	my @wd = split(/\s+/, $word);

	$in{pg} =~ s/\D//g;
	$in{pg} ||= 0;
	
	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';

	# 月次ファイル読み取り
	my @tmp;
	opendir(DIR,"$cf{logdir}");
	while( my $dir = readdir(DIR) ) {
		next if ($dir !~ /^(\d{6})\.txt$/);
		
		push(@tmp,$1);
	}
	closedir(DIR);
	
	# データ順次読み込み
	my ($i,$n,$flg,@find,%ym);
	for my $log ( sort{ $b <=> $a } @tmp ) {
		my $bw = File::ReadBackwards->new("$cf{logdir}/$log.txt") or next;
		while( defined( my $line = $bw->readline ) ) {
			my ($no,$d,$wk,$sub,$msg,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef) = split(/<>/,$line);

		my $flg;
		foreach my $wd (@wd) {
			if ("$sub $msg" =~ /^(?:$byte1|$byte2|$byte3|$byte4)*?\Q$wd\E/i) {
				$flg++;
				if ($in{cond} == 0) { last; }
			} else {
				if ($in{cond} == 1) { $flg = 0; last; }
			}
		}
		next if (!$flg);

			$i++;
			if ($cf{find_data} > 0) {
				next if ($i < $in{pg} + 1);
				next if ($i > $in{pg} + $cf{find_data});
			}

			$n++;
			$ym{$n} = $log;
			push(@find,$line);
		}
	}
	# 検索結果
	return ('ワード検索',$i,\@find,\%ym);
}

#-----------------------------------------------------------
#  カレンダ
#-----------------------------------------------------------
sub put_calen {
	my ($loop,$mday,$mon,$year,$cal) = @_;

	# 1日の曜日を求める
	my $week = get_week($in{y},$in{m},1);

	# 末日を求める
	my $last = last_day($in{y},$in{m});

	# 祝日を認識
	my $hdays = eval "getHolidays($in{y},$in{m},1);";

	my ($ret,$day,$end);
	foreach my $k (1 .. 6) {

		# 週のテンプレート情報
		my $tmp = $loop;

		# 末日のフラグが立った場合はループを終了
		last if ($end);

		# 日から土まで7日分をループ
		foreach my $i (0 .. 6) {

			# 第１週で初日の週より小のとき、又は終了フラグが立ったときは「空き枠」
			if (($k == 1 && $i < $week) || $end) {
				$tmp =~ s/!$i!/&nbsp;/;

			# 実枠あり
			} else {
				# 日を数える
				$day++;

				# 枠処理
				my $d = sprintf("%02d",$day);
				my $col  = defined $$hdays{$d} ? $cf{week_col}[7] : $cf{week_col}[$i];
				my $cday = defined($$cal{$d})
						? qq|<a href="$cf{diary_cgi}?mode=popup&amp;y=$in{y}&amp;m=$in{m}&amp;d=$day" class="lightview" data-lightview-type="iframe" data-lightview-options="width: 1100, height: 600" style="color:$col">$day</a>|
						: qq|<span style="color:$col">$day</span>|;
				
				# 本日は背景色を変える
				if ($in{y} == $year && $in{m} == $mon && $day == $mday) {
					$cday = qq|<div style="background:$cf{cal_today}">$cday</div>|;
				}

				$tmp =~ s/!$i!/$cday/;
			}

			# 末日に達した場合フラグを立てる
			if ($day >= $last) { $end = 1; }
		}
		$ret .= $tmp;
	}
	return $ret;
}

#-----------------------------------------------------------
#  前後の年月を求める
#-----------------------------------------------------------
sub get_ym {
	my ($y,$m) = @_;

	# 前月
	my ($b_y,$b_m);
	if ($m == 1) {
		$b_y = $y - 1;
		$b_m = 12;
	} else {
		$b_y = $y;
		$b_m = $m - 1;
	}
	# 翌月
	my ($n_y,$n_m);
	if ($m == 12) {
		$n_y = $y + 1;
		$n_m = 1;
	} else {
		$n_y = $y;
		$n_m = $m + 1;
	}
	return ($b_y,$b_m,$n_y,$n_m);
}

#-----------------------------------------------------------
#  ポップアップ画面
#-----------------------------------------------------------
sub popup {
	# 汚染チェック
	$in{y} =~ s/\D//g;
	$in{m} =~ s/\D//g;
	$in{d} =~ s/\D//g;

	# データファイル
	my $log = sprintf("%04d%02d",$in{y},$in{m});

	# 該当記事抽出
	my @log;
	open(IN,"$cf{logdir}/$log.txt") or error("open err: $log.txt");
	while (<IN>) {
		my ($d) = (split(/<>/))[1];

		if ($in{d} == $d) {
			push(@log,$_);
		}
	}
	close(IN);

	if (@log == 0) { error("該当記事なし"); }

	# テンプレート読込
	open(IN,"$cf{tmpldir}/popup.html") or error("open err: popup.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置き換え
	$tmpl =~ s/!cgi_title!/$cf{cgi_title}/g;
	$tmpl =~ s|!icon:(\w+)\.(\w+)!|<img src="$cf{iconurl}/$1.$2" alt="$1" class="icon" />|g;
	$tmpl =~ s/!year!/$in{y}/g;
	$tmpl =~ s/!month!/$in{m}/g;
	$tmpl =~ s/!day!/$in{d}/g;

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ /(.+)<!-- loop_begin -->(.+)<!-- loop_end -->(.+)/s
			? ($1,$2,$3)
			: error("テンプレート不正");

	# ヘッダ
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	# 表示開始
	foreach (@log) {
		my ($no,$d,$wk,$sub,$msg,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tube) = split(/<>/);

		# 自動リンク
		$msg = auto_link($msg) if ($cf{autolink});

	# YouTube
	if ($cf{tube}) {
		$tube =~ s|!([\w\-]{1,})!|<div class="imageCover"><a href="https://www.youtube.com/embed/$1" class="lightview" data-lightview-type="iframe" data-lightview-options="width: 845,height: 480, viewport: 'scale'"><img src="https://i.ytimg.com/vi/$1/mqdefault.jpg" width="240" height="154" class="baseImage"></a><a href="https://www.youtube.com/embed/$1" class="lightview" data-lightview-type="iframe" data-lightview-options="width: 845,height: 480, viewport: 'scale'"><img src="$cf{tube_ico}" width="64" height="64" class="coverImage"></a></div>|g;
	}

		# 画像
		my $image;
		if ($e1) { $image .= image(1,$no,$e1,$w1,$h1); }
		if ($e2) { $image .= image(2,$no,$e2,$w2,$h2); }
		if ($e3) { $image .= image(3,$no,$e3,$w3,$h3); }

		my $day = $d;
		$day =~ s/^0//;

		my $tmp = $loop;
		$tmp =~ s/!date!/$in{m}月$day日（$cf{week}[$wk]）/g;
		$tmp =~ s/!day-title!/$sub/g;
		$tmp =~ s/!comment!/$msg/g;
		$tmp =~ s/!image!/$image/g;
		$tmp =~ s/<!-- youtube -->/$tube/ if ($tube);
		print $tmp;
	}
	# フッタ
	footer($foot);
	#print $foot;
	exit;
}

#-----------------------------------------------------------
#  フッター
#-----------------------------------------------------------
sub footer {
	my $foot = shift;

	# 著作権表記（削除・改変禁止）
	my $copy = <<EOM;
<p style="margin-top:2em;text-align:center;font-size:11pt;" class="copy">
- <a href="http://www.kent-web.com/" target="_top">$cf{version}</a> -<br />
- <a href="http://drc3.news-site.net/script1/" target="_top">Edited by TRANSFORM</a> -
</p>
EOM

	if ($foot =~ /(.+)(<\/body[^>]*>.*)/si) {
		print "$1$copy$2\n";
	} else {
		print "$foot$copy\n";
		print "</body></html>\n";
	}
	exit;
}

#-----------------------------------------------------------
#  自動リンク
#-----------------------------------------------------------
sub auto_link {
	my $text = shift;

	$text =~ s/(s?https?:\/\/([\w-.!~*'();\/?:\@=+\$,%#]|&amp;)+)/<a href="$1" target="_blank">$1<\/a>/g;
	return $text;
}

#-----------------------------------------------------------
#  エラー画面
#-----------------------------------------------------------
sub error {
	my $err = shift;

	open(IN,"$cf{tmpldir}/error.html") or die;
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s/!error!/$err/g;

	print "Content-type: text/html; charset=shift_jis\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  繰越ボタン作成
#-----------------------------------------------------------
sub make_pgbtn {
	my ($i,$pg) = @_;

	# ページ繰越定義
	$cf{pg_max} ||= 10;
	my $next = $pg + $cf{pg_max};
	my $back = $pg - $cf{pg_max};

	# ページ繰越ボタン作成
	my $pg_btn;
	if ($back >= 0 || $next < $i) {
		$pg_btn .= "Page: ";

		my ($x, $y) = (1, 0);
		while ($i > 0) {
			if ($pg == $y) {
				$pg_btn .= qq(| <b>$x</b> );
			} else {
				$pg_btn .= qq(| <a href="$cf{diary_cgi}?mode=$in{mode}&amp;y=$in{y}&amp;m=$in{m}&amp;pg=$y">$x</a> );
			}
			$x++;
			$y += $cf{pg_max};
			$i -= $cf{pg_max};
		}
		$pg_btn .= "|";
	}
	return $pg_btn;
}

#-----------------------------------------------------------
#  アルバムページ送り作成
#-----------------------------------------------------------
sub album_pager {
	my ($i,$pg,$stat) = @_;
	
	# 引数
	my $param = $in{y} && $in{m} ? "&amp;y=$in{y}&amp;m=$in{m}" : '';
	
	# ページ繰越数定義
	my $next = $pg + $cf{album_data};;
	my $back = $pg - $cf{album_data};
	
	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><span>$x</span></li>\n!);
			} else {
				push(@pg,qq!<li><a href="$cf{bbs_cgi}?$stat&pg=$y$param">$x</a></li>\n!);
			}
			$x++;
			$y += $cf{album_data};
			$z -= $cf{album_data};
			
			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$cf{calen_cgi}?$stat&pg=$back$param">prev</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$cf{calen_cgi}?$stat&pg=$next$param">next</a></li>\n!;
	}
	
	# 結果を返す
	return $ret ? qq|<ul class="pager">\n$ret</ul>| : '';
}

#-----------------------------------------------------------
#  ワード検索ページ送り作成
#-----------------------------------------------------------
sub find_pager {
	my ($i,$pg,$stat) = @_;
	
	# ページ繰越数定義
	my $next = $pg + $cf{find_data};;
	my $back = $pg - $cf{find_data};
	
	my $word = url_enc($in{word});

	# パラメータ
	my $param = "&word=$word&cond=$in{cond}";

	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><span>$x</span></li>\n!);
			} else {
				push(@pg,qq!<li><a href="$cf{bbs_cgi}?$stat&pg=$y$param">$x</a></li>\n!);
			}
			$x++;
			$y += $cf{find_data};
			$z -= $cf{find_data};
			
			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$cf{calen_cgi}?$stat&pg=$back$param">prev</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$cf{calen_cgi}?$stat&pg=$next$param">next</a></li>\n!;
	}
	
	# 結果を返す
	return $ret ? qq|<ul class="pager">\n$ret</ul>| : '';
}

#-----------------------------------------------------------
#  画像表示
#-----------------------------------------------------------
sub image {
	my ($i,$no,$ex,$w,$h,$sub) = @_;

	my $image;
	if (-f "$cf{imgdir}/$no-s-$i$ex") {
		$image = qq|<img src="$cf{imgurl}/$no-s-$i$ex" class="snapshot" alt="" />|;

	} else {
		($w,$h) = resize($w,$h);
		$image = qq|<div class="snapshot"><img src="$cf{imgurl}/$no-$i$ex" width="$w" height="$h" class="baseImage" alt="" /><img src="$cf{img_ico}" width="24" height="24" class="coverImage"></div>|;
		#$image = qq|<img src="$cf{imgurl}/$no-$i$ex" width="$w" height="$h" class="snapshot" alt="" />|;
	}
	return qq|<a href="$cf{imgurl}/$no-$i$ex" class="lightview" data-lightview-title="$sub" data-lightview-group-options="skin: 'mac'">$image</a>\n|;
}

#-----------------------------------------------------------
#  アルバム画像表示
#-----------------------------------------------------------
sub album_image {
	my ($i,$no,$ex,$w,$h,$sub) = @_;
	
	my $image;
	if (-f "$cf{imgdir}/$no-s-$i$ex") {
		$image = qq|<img src="$cf{imgurl}/$no-s-$i$ex" class="img" alt="" />|;
	} else {
		($w,$h) = resize($w,$h);
		$image = qq|<img src="$cf{imgurl}/$no-$i$ex" width="$w" height="$h" class="img" alt="" />|;
	}
	return qq|<a href="$cf{imgurl}/$no-$i$ex" class="lightview" data-lightview-title="$sub" data-lightview-group="gbook-set" data-lightview-group-options="skin: 'mac'">$image</a>\n|;
}

#-----------------------------------------------------------
#  URLエンコード
#-----------------------------------------------------------
sub url_enc {
	local($_) = @_;

	s/(\W)/'%' . unpack('H2', $1)/eg;
	s/\s/+/g;
	$_;
}
